# files of nvboard
NBD_SRC = $(NVBOARD_HOME)/src
NBD_SRCS := $(shell find $(NBD_SRC) -name "*.cpp")
NBD_LIB = $(NVBOARD_HOME)/include
INC_PATH += $(NBD_LIB)

NBD_DST_DIR = $(NVBOARD_HOME)/build
NBD_OBJS := $(addprefix $(NBD_DST_DIR)/, $(addsuffix .o, $(basename $(notdir $(NBD_SRCS)))))

# The archive of nvboard
NBD_ARCHIVE = $(NBD_DST_DIR)/NVBOARD.a
CXXFLAGS += -MMD -O3

$(NBD_DST_DIR)/%.o: $(NBD_SRC)/%.cpp
	@echo + CXX "->" NVBOARD_HOME/$(shell realpath $< --relative-to $(NVBOARD_HOME))
	@mkdir -p $(dir $@)
	@$(OBJCACHE) $(CXX) $(CXXFLAGS) -I$(NBD_LIB) -c -o $@ $<

# Build the archive of nvboard
$(NBD_ARCHIVE): $(NBD_OBJS)
	@echo + AR "->" $(shell realpath $@ --relative-to $(NVBOARD_HOME))
	@ar rcs $(NBD_ARCHIVE) $(NBD_OBJS)

# Rule (`#include` dependencies): paste in `.d` files generated by gcc on `-MMD`
-include $(NBD_OBJS:.o=.d)

.PHONY: archive nbd-clean

archive: $(NBD_ARCHIVE) 

nbd-clean:
	rm -rf $(NBD_DST_DIR)
